==> python_glowplug-0.2.3-py3-none-any.whl: glowplug/mssql.py <==
from sqlalchemy import text
from sqlalchemy.ext.asyncio import create_async_engine
from .base import DbDriver


class MsSqlDriver(DbDriver):
    """Microsoft SQL Server database driver."""

    path: str

    maintenance_db: str

    def __init__(self, path: str, maintenance_db: str = "master", **kwargs):
        """Initialize the driver.

        Args:
            path (str): The path to connect to the database.
            maintenance_db (str, optional): The database to connect to for maintenance. Defaults to "master".
        """
        super().__init__(**kwargs)
        self.path = path
        self.maintenance_db = maintenance_db

    async def exists(self) -> bool:
        """Check if the database exists."""
        # Get a database-less path and the db name
        path, database = self._split_path()
        # Connect to the database-less path
        engine = create_async_engine(f"mssql+aioodbc://{path}")
        async with engine.connect() as conn:
            # Check if the database exists
            result = await conn.execute(
                text("SELECT name FROM sys.databases WHERE name = :db"),
                {"db": database},
            )
            row = await result.fetchone()
            return row is not None

    async def create(self) -> None:
        """Create the database."""
        # Get a database-less path and the db name
        path, database = self._split_path()
        # Connect to the database-less path
        engine = create_async_engine(f"mssql+aioodbc://{path}")
        # Create a raw connection with autocommit on.
        # See: https://stackoverflow.com/a/42008664
        conn = await engine.raw_connection()
        conn.driver_connection.autocommit = True
        await conn.driver_connection.execute(f"CREATE DATABASE {database}")
        conn.close()

    @property
    def async_uri(self) -> str:
        """Connect with aioodbc."""
        return f"mssql+aioodbc://{self.path}"

    @property
    def sync_uri(self) -> str:
        """Connect with pyodbc."""
        return f"mssql+pyodbc://{self.path}"

    def _split_path(self) -> tuple[str, str]:
        """Extract the database name from the rest of the path."""
        # Split the path into everything before the / and after
        head, tail = self.path.split("/", 1)
        # Extract the database name from the tail
        database, query = tail.split("?", 1)
        # Re-join the query to the beginning of the path, with the
        # name of the maintenance database.
        path = f"{head}/{self.maintenance_db}?{query}"
        return path, database

==> python_glowplug-0.2.4-py3-none-any.whl: glowplug/mssql.py <==
from sqlalchemy import text
from sqlalchemy.ext.asyncio import create_async_engine
from .base import DbDriver


class MsSqlDriver(DbDriver):
    """Microsoft SQL Server database driver."""

    path: str

    maintenance_db: str

    def __init__(self, path: str, maintenance_db: str = "master", **kwargs):
        """Initialize the driver.

        Args:
            path (str): The path to connect to the database.
            maintenance_db (str, optional): The database to connect to for maintenance. Defaults to "master".
        """
        super().__init__(**kwargs)
        self.path = path
        self.maintenance_db = maintenance_db

    async def exists(self) -> bool:
        """Check if the database exists."""
        # Get a database-less path and the db name
        path, database = self._split_path()
        # Connect to the database-less path
        engine = create_async_engine(f"mssql+aioodbc://{path}")
        async with engine.connect() as conn:
            # Check if the database exists
            result = await conn.execute(
                text("SELECT name FROM sys.databases WHERE name = :db"),
                {"db": database},
            )
            row = result.fetchone()
            return row is not None

    async def create(self) -> None:
        """Create the database."""
        # Get a database-less path and the db name
        path, database = self._split_path()
        # Connect to the database-less path
        engine = create_async_engine(f"mssql+aioodbc://{path}")
        # Need to run this with autocommit=True to avoid a transaction error
        # See: https://stackoverflow.com/a/42008664
        async with engine.connect() as conn:
            await conn.execute(
                text(f"CREATE DATABASE {database}").execution_options(autocommit=True)
            )

    @property
    def async_uri(self) -> str:
        """Connect with aioodbc."""
        return f"mssql+aioodbc://{self.path}"

    @property
    def sync_uri(self) -> str:
        """Connect with pyodbc."""
        return f"mssql+pyodbc://{self.path}"

    def _split_path(self) -> tuple[str, str]:
        """Extract the database name from the rest of the path."""
        # Split the path into everything before the / and after
        head, tail = self.path.split("/", 1)
        # Extract the database name from the tail
        database, query = tail.split("?", 1)
        # Re-join the query to the beginning of the path, with the
        # name of the maintenance database.
        path = f"{head}/{self.maintenance_db}?{query}"
        return path, database
